@@ -1,14 +1,9 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 //
-// This file is part of the Marble Virtual Globe.
-//
-// This program is free software licensed under the GNU LGPL. You can
-// find a copy of this license in LICENSE.txt in the top directory of
-// the source code.
-//
-// Copyright 2006-2007 Torsten Rahn      <tackat@kde.org>
-// Copyright 2007      Inge Wallin       <ingwa@kde.org>
-// Copyright 2012      Illya Kovalevskyy <illya.kovalevskyy@gmail.com>
-// Copyright 2014      Gábor Péterffy    <peterffy95@gmail.com>
+// SPDX-FileCopyrightText: 2006-2007 Torsten Rahn <tackat@kde.org>
+// SPDX-FileCopyrightText: 2007 Inge Wallin <ingwa@kde.org>
+// SPDX-FileCopyrightText: 2012 Illya Kovalevskyy <illya.kovalevskyy@gmail.com>
+// SPDX-FileCopyrightText: 2014 Gábor Péterffy <peterffy95@gmail.com>
 //
 
 // Self
@@ -77,6 +72,7 @@ public:
     QAction *m_directionsToHereAction;
 
     QAction *const m_copyCoordinateAction;
+    QAction *const m_copyGeoAction;
 
     QAction *m_rmbExtensionPoint;
 
@@ -110,10 +106,11 @@ MarbleWidgetPopupMenu::Private::Private( MarbleWidget *widget, const MarbleModel
     m_widget(widget),
     m_lmbMenu( m_widget ),
     m_rmbMenu( m_widget ),
-    m_directionsFromHereAction( 0 ),
-    m_directionsToHereAction( 0 ),
+    m_directionsFromHereAction( nullptr ),
+    m_directionsToHereAction( nullptr ),
     m_copyCoordinateAction(new QAction(QIcon(QStringLiteral(":/icons/copy-coordinates.png")), tr("Copy Coordinates"), parent)),
-    m_rmbExtensionPoint( 0 ),
+    m_copyGeoAction(new QAction(QIcon(QStringLiteral(":/icons/copy-coordinates.png")), tr("Copy geo: URL"), parent)),
+    m_rmbExtensionPoint( nullptr ),
     m_runnerManager( model )
 {
     // Property actions (Left mouse button)
@@ -150,6 +147,7 @@ MarbleWidgetPopupMenu::Private::Private( MarbleWidget *widget, const MarbleModel
     m_rmbMenu.addAction( addBookmark );
     if ( !smallScreen ) {
         m_rmbMenu.addAction( m_copyCoordinateAction );
+        m_rmbMenu.addAction( m_copyGeoAction );
     }
     m_rmbMenu.addAction(QIcon(QStringLiteral(":/icons/addressbook-details.png")), tr("&Address Details"), parent, SLOT(startReverseGeocoding()));
     m_rmbMenu.addSeparator();
@@ -167,6 +165,7 @@ MarbleWidgetPopupMenu::Private::Private( MarbleWidget *widget, const MarbleModel
     parent->connect( addBookmark, SIGNAL(triggered()), SLOT(addBookmark()) );
     parent->connect( aboutDialogAction, SIGNAL(triggered()), SLOT(slotAboutDialog()) );
     parent->connect( m_copyCoordinateAction, SIGNAL(triggered()), SLOT(slotCopyCoordinates()) );
+    parent->connect( m_copyGeoAction, SIGNAL(triggered()), SLOT(slotCopyGeo()) );
     parent->connect( m_infoDialogAction, SIGNAL(triggered()), SLOT(slotInfoDialog()) );
     parent->connect( fullscreenAction, SIGNAL(triggered(bool)), parent, SLOT(toggleFullscreen(bool)) );
 
@@ -578,7 +577,7 @@ void MarbleWidgetPopupMenu::showLmbMenu( int xpos, int ypos )
         d->m_infoDialogAction->setText( name );
         d->m_infoDialogAction->setIcon( icon );
         // Insert as first action in the menu
-        QAction *firstAction = 0;
+        QAction *firstAction = nullptr;
         if( !d->m_lmbMenu.actions().isEmpty() ) {
             firstAction = d->m_lmbMenu.actions().first();
         }
@@ -623,6 +622,7 @@ void MarbleWidgetPopupMenu::showRmbMenu( int xpos, int ypos )
 
     QPoint curpos = QPoint( xpos, ypos );
     d->m_copyCoordinateAction->setData( curpos );
+    d->m_copyGeoAction->setData( curpos );
 
     bool const showDirectionButtons = d->m_widget->routingLayer() && d->m_widget->routingLayer()->isInteractive();
     d->m_directionsFromHereAction->setVisible( showDirectionButtons );
@@ -644,7 +644,7 @@ void MarbleWidgetPopupMenu::resetMenu()
 void MarbleWidgetPopupMenu::slotInfoDialog()
 {
     QAction *action = qobject_cast<QAction *>( sender() );
-    if ( action == 0 ) {
+    if ( action == nullptr ) {
         mDebug() << "Warning: slotInfoDialog should be called by a QAction signal";
         return;
     }
@@ -711,7 +711,7 @@ void MarbleWidgetPopupMenu::slotInfoDialog()
         }
 
         if ( placemark ) {
-            if ( placemark->style() == 0 ) {
+            if ( placemark->style() == nullptr ) {
                 popup->setBackgroundColor(QColor(Qt::white));
                 popup->setTextColor(QColor(Qt::black));
                 return;
@@ -791,6 +791,22 @@ void MarbleWidgetPopupMenu::slotCopyCoordinates()
     }
 }
 
+void MarbleWidgetPopupMenu::slotCopyGeo()
+{
+    const GeoDataCoordinates coordinates = d->mouseCoordinates( d->m_copyCoordinateAction );
+    if ( coordinates.isValid() ) {
+        const qreal latitude_degrees = coordinates.latitude(GeoDataCoordinates::Degree);
+        const qreal longitude_degrees = coordinates.longitude(GeoDataCoordinates::Degree);
+
+        QMimeData * const myMimeData = new QMimeData();
+        QList<QUrl> urls = { QUrl(QString("geo:%1,%2").arg(latitude_degrees, 0, 'f', 10).arg(longitude_degrees, 0, 'f', 10)) };
+        myMimeData->setUrls(urls);
+        QClipboard * const clipboard = QApplication::clipboard();
+        clipboard->setMimeData(myMimeData);
+    }
+}
+
+
 void MarbleWidgetPopupMenu::slotAboutDialog()
 {
     QPointer<MarbleAboutDialog> dialog = new MarbleAboutDialog( d->m_widget );
